from src.logs import Log
from src.settings import Settings
from src.sql.models.pschitt import PschittUserBase, PschittUserModel, PschittBase
from src.sql.schemas.pschitt import Pschitt, PschittUser
from src.sql.database import SessionLocalRo, SessionLocalRw
from sqlalchemy import desc, func, text

log = Log("crud_PschittUser")
config = Settings().get_config()


class PschittUserCrud:
    @staticmethod
    def get_user_by_id(user_id: int):
        """get_user_by_id.

        :param user_id:
        :type user_id: int
        """
        with SessionLocalRo() as session:
            return session.query(PschittUser).filter(id == user_id).first()
        return user

    @staticmethod
    def get_all_users():
        """get_all_users"""
        with SessionLocalRo() as session:
            # req = session.query(func.count(PschittUser.pschitt).group_by(PschittUser.id))
            req = (
                session.query(PschittUser, func.count(Pschitt.datetime).label("total"))
                .join(Pschitt)
                .group_by(PschittUser.id)
                .order_by(text("total DESC"))
                .all()
            )
            return req

    @staticmethod
    def get_user(user: PschittUserBase):
        """get_user.

        :param user:
        :type user: PschittUserBase
        """
        with SessionLocalRo() as session:
            return (
                session.query(PschittUser)
                .filter(PschittUser.user_id == user.user_id)
                .first()
            )
        return None

    @staticmethod
    def get_users():
        return user

    @staticmethod
    def create_user(user: PschittUserBase):
        user_schema = PschittUser(user_id=user.user_id, guild_id=user.guild_id)
        with SessionLocalRw() as session:
            session.add(user_schema)
            session.commit()
        return True

    @staticmethod
    def update_user(user: PschittUserModel):
        with SessionLocalRw() as session:
            biketag_user = (
                session.query(PschittUser)
                .filter(PschittUser.user_id == user.id)
                .first()
            )
            biketag_user.point = user.point
            session.commit()
            session.refresh(biketag_user)
            return biketag_user
        return

    @staticmethod
    def delete_user(user: PschittUserBase):
        return user
