import pendulum
from sqlalchemy import Column, BigInteger, DateTime, ForeignKey, Table
from sqlalchemy.orm import Mapped, mapped_column, relationship
from sqlalchemy.sql import func
from typing import List, Optional

from src.sql.database import Base

# pschitt_association_table = Table(
#    "pschitt",
#    Base.metadata,
#    Column("pschitt_user_id", ForeignKey("pschitt_user.id")),
#    Column("pschitt_datetime_id", ForeignKey("pschitt_datetime.id")),
# )

# association_table = Table(
#    "association_table",
#    Base.metadata,
#    Column("pschitt_user_id", ForeignKey("pschitt_user.id"), primary_key=True),
#    Column("pschitt_datetime_id", ForeignKey("pschitt_datetime.id"), primary_key=True),
# )

# class PschittAssociation(Base):
#    """PschittAssociation"""
#
#    __tablename__ = "pschitt"
#    __allow_unmapped__ = True
#    Base.metadata,
#    pschitt_user_id: Column(BigInteger, ForeignKey("pschitt_user.id"), primary_key=True)
#    pschitt_datetime_id: Column(BigInteger, ForeignKey("pschitt_datetime.id"), primary_key=True)


class Pschitt(Base):
    """Pschitt"""

    __tablename__ = "pschitt_datetime"
    id: Mapped[int] = mapped_column(primary_key=True)
    pschitt_user_id: Mapped[int] = mapped_column(ForeignKey("pschitt_user.id"))
    datetime = Column(
        "date", DateTime(timezone=True), server_default=func.now(), nullable=False
    )
    pschitt_user: Mapped[List["PschittUser"]] = relationship(back_populates="pschitt")


class PschittUser(Base):
    """Pschitt"""

    __tablename__ = "pschitt_user"
    id: Mapped[int] = mapped_column(primary_key=True)
    user_id = Column(BigInteger, nullable=False, unique=True)
    guild_id = Column(BigInteger, nullable=False)
    pschitt: Mapped[List["Pschitt"]] = relationship(
        back_populates="pschitt_user", lazy="immediate"
    )
